<?php
GFForms::include_feed_addon_framework();

class GFSalesmate extends GFFeedAddOn {

    protected $_version = GF_SALESMATE_VERSION;
    protected $_min_gravityforms_version = '1.9.16';
    protected $_slug = 'gravityformsalesmate';
    protected $_path = 'gravityformsalesmate/salesmate.php';
    protected $_full_path = __FILE__;
    protected $_title = 'Gravity Forms Salesmate Add-On';
    protected $_short_title = 'Salesmate';
    private static $_instance = null;
    protected $_valid_api_key;
    protected $default_stages;

    /**
     * Get an instance of this class.
     *
     * @return GFSimpleAddOn
     */
    public static function get_instance() {

        if (self::$_instance == null) {

            self::$_instance = new GFSalesmate();

        }
        return self::$_instance;
    }

    /*
     * Plugin starting point. Handles hooks, loading of language files and PayPal delayed payment support.
    */
    public function init() {
      parent::init();
      $this->add_delayed_payment_support(
            array(
                'option_label' => esc_html__('Subscribe contact to service x only when payment is received.', 'gravityformsalesmate')
            )
      );

      //New salesmate menu page for the error log
      add_filter( 'gform_addon_navigation', array($this,'logMenu') , 30);
      add_action( 'wp_ajax_remove_logs', array( $this, 'remove_logs' ),10,2 );
      add_action( 'wp_ajax_nopriv_remove_logs', array( $this, 'remove_logs' ),10,2 );
      add_action( 'wp_ajax_process_retry', array( $this, 'process_retry' ),10,2 );
      add_action( 'wp_ajax_nopriv_process_retry', array( $this, 'process_retry' ),10,2 );
      add_action( 'wp_ajax_remove_alllogs', array( $this, 'remove_alllogs' ),10,2 );
      add_action( 'wp_ajax_nopriv_remove_alllogs', array( $this, 'remove_alllogs' ),10,2 );
    }
    
       
    /*
    * Callback function of clear all logs
    * @return boolean
    */
    public function remove_alllogs(){
      global $wpdb;
      $delete = $wpdb->query("TRUNCATE TABLE $wpdb->prefix".'salesmatelogs');
      echo $delete;
      exit(0);
  }

  /*
  * Callback for ajaxification for retry log data
  * @return numbers for particular action
  */
  public function process_retry(){
      global $wpdb;
      $data = $wpdb->get_results("SELECT trieddata,fromodule FROM $wpdb->prefix".'salesmatelogs where id='.sanitize_text_field($_POST['myID']));

      $resend=$data[0]->trieddata;
      $modules = $data[0]->fromodule;

      if($resend!="" && $modules!="" ){

        switch ($data[0]->fromodule) {

          case 'Contacts':
              $rslt=$this->doRequest('contact','POST',$resend,'contact');
                if(is_object($rslt)){
                  if($rslt->Status == "success"){
                    $removeqry = $wpdb->delete( $wpdb->prefix.'salesmatelogs', array( 'id' => sanitize_text_field($_POST['myID']) ) );
                    echo $removeqry;
                  }
                }else {
                   $err = json_decode($rslt);
                   if($err->Status == 'failure'){
                     echo 2;
                   }
                }
            break;
          case 'Deal':
              $deal = $this->doRequest('deal','POST',$resend,'save');
              if(is_object($deal)){
                $removeqry = $wpdb->delete( $wpdb->prefix.'salesmatelogs', array( 'id' => sanitize_text_field($_POST['myID']) ) );
                echo $removeqry;
              }else {
                $err = json_decode($deal);
                if($err->Status == 'failure'){
                  echo 2;
                }
              }
          default:

            break;

        }

      }else {
        echo 4;
      }
      exit(0);
  }

  
    /*
    * Callback of ajaxify hook for remove particular error log
    * @return boolean
    */
    public  function remove_logs(){
      global $wpdb;
      $removeerrqry = $wpdb->delete( $wpdb->prefix.'salesmatelogs', array( 'id' => sanitize_text_field($_POST['myID']) ) );
      echo $removeerrqry;
      exit(0);
  }

  /*
  *callback function for menu page in gravity form
  *@return Array
  */
  function logMenu( $menus ) {
      $menus[] = array( 'name' => 'salesmatelog_detail', 'label' => __( 'Salesmate logs' ), 'callback' =>  array($this,'salesmatelog_detail') );
      return $menus;
  }
  
    /*
    * Log page callback function
    * @return nothing
    */
    function salesmatelog_detail() {

      global $wpdb;
      $raw = $wpdb->get_results("SELECT * FROM $wpdb->prefix".'salesmatelogs');
      echo '<form method="post" class="logform" action="" enctype="multipart/form-data">';
      echo '<div class="cf7_salesmate_form">';
      echo '<h1 class="logtitle">Failed salesmate logs</h1>';
      echo '<div class="logData">';
      if(count($raw)>0) {

          echo '<table>';
          echo '<thead>';
          echo '<tr>';
          echo '<th><span>Index</span></th>';
          echo '<th><span>Form Id</span></th>';
          echo '<th><span>Module</span></th>';
          echo '<th><span>Detail</span></th>';
          echo '<th><span>Sended Data</span></th>';
          echo '<th><span>Generated At</span></th>';
          echo '<th><span>Operations</span></th>';
          echo '</tr>';
          echo '</thead>';
          echo '<tbody>';
          $i=1;
          foreach ($raw as $datas) {
             echo '<tr>';
             echo '<td>'.$i.'</td>';
             echo '<td>'.$datas->form_id.'</td>';
             echo '<td>'.$datas->fromodule.'</td>';
             echo '<td><textarea cols="50" rows="4">'.$datas->errlogs.'</textarea></td>';
             echo '<td><textarea class="sendeddata" cols="50" rows="4">'.$datas->trieddata.'</textarea></td>';
             echo '<td>'.$datas->logat.'</td>';
             echo '<td><input type="button" class="button-secondary retry" data-index="'.$datas->id.'" value="Retry"/>&nbsp;<input type="button" class="button-secondary remove" value="Delete" data-index="'.$datas->id.'"/></td>';
             echo '</tr>';
             $i++;
          }
          echo '</tbody>';
          echo'</table>';

      }else{

          echo '<div class="nodata"><span>No Logs Available</span></div>';

      }

      echo '</div>';
      if(count($raw)>0) {
          echo '<input type="button" name="Submit" class="button-primary logclear" value="Clear Logs" />';
      }
      echo '</form>';

  }

  /*
  * Prefix domain for Make API call
  * @return string URL
  */
  public function getApiDomain($endpoint, $type='', $method='') {
      $all_settings = $this->get_plugin_settings();
      if( $type == 'search'  ){
          return $all_settings['domainname'].'/apis/'.$endpoint.'/v4/'.$type;
      } else if( $type == '' || $type == 'Owners' || ( $type == 'contact' && $method == 'PUT' )){
        return $all_settings['domainname'].'/apis/'.$endpoint;
      } else if( $type == 'save' || $type == 'contact' ){
          return $all_settings['domainname'].'/apis/'.$endpoint.'/v4';
      } else if( $type == 'currency' ){
        return $all_settings['domainname'].'/apis/v3/'.$endpoint;
      }  else{
          return $all_settings['domainname'].'/apis/v1/'.$endpoint;
      }
  }

  /*
  * Required configuration for access API
  * @return Array of required configuration for Salesmate API
  */
  public function header_data() {

      $get_settings = $this->get_plugin_settings();
      $domainname = str_replace('https://','',$get_settings['domainname']);
      $check_header_data = array(
        "Content-Type:application/json",
        "accessToken:".$get_settings['sessiontoken'],
        "x-linkname:".$domainname
      );
    return $check_header_data;
  }

  public function createNote($contactId,$posted_data,$module) {
    
    $note_contact="";
    $note_contact.="<p><br>";
    $note_contact.='<b>Contact Form Submission:</b><br><br>';
    foreach ($posted_data as $key => $value) {
      if(!is_array($value)){
        if($value!=''){
          $note_contact.='<b>'.str_replace("-", " ",preg_replace('/[^A-Za-z0-9\-]/', '',ucfirst(str_replace(" ", "-" ,$key)))).': </b>'.$value.'<br>';
        }else {
          $note_contact.='<b>'.str_replace("-", " ",preg_replace('/[^A-Za-z0-9\-]/', '',ucfirst(str_replace(" ", "-" ,$key)))).': </b> - <br>';
        }
      }else {
        $multivaluenote="";
        $j=0;
        for($i=0; $i<count($value); $i++){
            $multivaluenote.= ($j+1).'.'.$value[$i].'<br>';
        }
        $note_contact.='<b>'.str_replace("-", " ",preg_replace('/[^A-Za-z0-9\-]/', '',ucfirst(str_replace(" ", "-" ,$key)))).': </b>'.$multivaluenote.'<br>';
      }
    }
    $note_contact.="</p>";
    // $note_contact="<p><b>Contact Note</b><br>ID:'".($search->Data->data[0]->id!='')?$search->Data->data[0]->id:'-'."<br>";
    $noteCollection = array();
    $createnote = '{"note":"'.$note_contact.'"}';

    if($module == 'contact'){
      $noteresponse = $this->doRequest('modules/1/object/'.$contactId.'/notes','POST',$createnote,'addnote');
    }else {
      $noteresponse = $this->doRequest('modules/4/object/'.$contactId.'/notes','POST',$createnote,'addnote');
    }
  }

  public function createContact( $postdata ){
    unset($postdata['textCustomField8']);
    unset($postdata['emailOptOut']);
    unset($postdata['emailOptOutReason']);
    unset($postdata['smsOptOut']);
    
    $endpoint = 'contact';
    $rslt=$this->doRequest($endpoint,'POST',json_encode($postdata),'contact');
    
    if(!is_object($rslt)){
      $err = json_decode($rslt);
      if($err->Status == 'failure'){
        echo '2';
      }
    }
  }

  public function updateContact( $postdata, $id ){
    unset($postdata['textCustomField8']);
    unset($postdata['emailOptOut']);
    unset($postdata['emailOptOutReason']);
    unset($postdata['smsOptOut']);

    $tags = $postdata['tags'];
    unset($postdata['tags']);
    
    $tags_endpoint = 'contact/v4/'.$id;
    $tags_rslt=$this->doRequest($tags_endpoint,'GET');
    if(isset($tags_rslt->Data)){
       $postdata['tags'] = $tags_rslt->Data->tags.','.$tags;
    }
   

    $endpoint = 'contact/v4/'.$id;
    $rslt=$this->doRequest($endpoint,'PUT',json_encode($postdata),'contact');
    if(!is_object($rslt)){
      $err = json_decode($rslt);
      if(!empty($err->Status) && $err->Status == 'failure'){
        echo '2';
      }
    }
  }

	/**
	 * Process the feed e.g. subscribe the user to a list.
	 *
	 * @param array $feed The feed object to be processed.
	 * @param array $entry The entry object currently being processed.
	 * @param array $form The form object currently being processed.
	 *
	 * @return bool|void
	 */
	public function process_feed( $feed, $entry, $form ) {
		$formID = $form['fields'][0]->formId;
        $feedName = $feed['meta']['feedName'];
		
        $owner=$feed['meta']['owner'];

        $deal_status=$feed['meta']['deal'];

        $deal_stage=$feed['meta']['stage'];

        $deal_pipeline=$feed['meta']['pipeline'];

        $deal_winprobability=$feed['meta']['winProbability'];

        $deal_priority=$feed['meta']['priority'];

        $deal_source=$feed['meta']['source'];

        $deal_currency=$feed['meta']['currency'];

        $record=$feed['meta']['record'];
        
        if( $record == "" ){
          $record = 3;
        }
        
        $deal_primarycontact_id="";
        $company_id = '';
        $end_point = 'contact';
        $field_map = $this->get_field_map_fields($feed, 'mappedFields');

        $email="";
        $naming="";

        /*Front Form Data*/
        $flds = $form['fields'];
        $noteFields = array();
         foreach ($flds as $data) {
            if($data->type == 'name' || $data->type == 'checkbox') {
               for($i=0; $i<count($data->inputs); $i++){
                  $noteFields[$data->inputs[$i]['label']] = $entry[$data->inputs[$i]['id']];
               }
            }else {
              if($data->id != ''){
                if(isset($entry[$data->id])){
                  $noteFields[$data->label] = $entry[$data->id];
                }
              }
            }
         }
        $noteFields['Form title'] = $form['title'];


        // Loop through the fields from the field map setting building an array of values to be passed to the third-party service.
        $chk_arr=array();
        $merge_vars = array();
        $company_vars=array();
        $company_new=array();
        foreach ($field_map as $name => $field_id) {

            // Get the field value for the specified field id

            if($name=='email'){

                $email=$this->get_field_value($form, $entry, $field_id);
            }

            if($name=='name'){
              if($this->get_field_value($form, $entry, $field_id)==""){
                $merge_vars[$name]="";
                $naming="No name";
              }else {
                $merge_vars[$name]=$this->get_field_value($form, $entry, $field_id);
                $naming=$this->get_field_value($form, $entry, $field_id);
              }
            }

            if($name=='lastName'){
              if($this->get_field_value($form, $entry, $field_id)==""){
                $merge_vars[$name]=$naming;
              }else {
                $merge_vars[$name]=$this->get_field_value($form, $entry, $field_id);
              }
            }

            if ($name == 'website') {

                if (!filter_var($this->get_field_value($form, $entry, $field_id), FILTER_VALIDATE_URL) === false) {
                    $merge_vars[$name] = esc_url($this->get_field_value($form, $entry, $field_id));
                } else {
                    $merge_vars[$name] = '';
                }

            } else {
                $merge_vars[$name] = $this->get_field_value($form, $entry, $field_id);
            }
            
            /* This portion is for company field */
            if($name=='company')
            {
                if($this->get_field_value($form, $entry, $field_id))
                {
                    //changing the search functionality 28/6/2018
                    $seachCompanydata = '{"fields": ["name","website"],"filterQuery": {"group": {"operator": "AND","rules": [{"condition": "EQUALS","moduleName": "Company","field": {"fieldName": "name"},"data": "'.$this->get_field_value($form, $entry, $field_id).'"}]}}}';
                    $company_vars = $this->doRequest('company','POST',$seachCompanydata,'search');
                    if(is_object($company_vars)){

                        if(count($company_vars->Data)>0)
                        {
                            $company_id=$company_vars->Data->id;

                        }
                        $merge_vars[$name] = $company_id;
                    }else{

                        $company_new['name'] = $this->get_field_value($form, $entry, $field_id);
                        $company_new['owner'] = $owner;
                        $company_data = json_encode($company_new);
                        $new_company = $this->doRequest('company','POST',$company_data,'save');
                        $company_id = $new_company->Data->id;
                        $merge_vars[$name] = $company_id;
                    }
                }else{

                    $company_vars='no data';

                }
            }

        }

        $merge_vars['owner'] = (int)$owner; //default value to set
        $send_salesmate_data = json_encode($merge_vars);

        // Update contacts search and duplication function

    $search='{"displayingFields":["contact.company.name","contact.email","contact.id","contact.mobile","contact.name","contact.lastName"],"filterQuery":{"group":{"operator":"AND","rules":[{"condition":"EQUALS","moduleName":"Contact","field":{"fieldName":"contact.email","displayName":"Email"},"data":"'.$email.'","eventType":"Email"}]}},"sort":{"fieldName":"","order":""},"moduleId":1,"reportType":"get_data","getRecordsCount":true}';
        $srch=$this->doRequest('contact','POST',$search,'search');
      
        if(is_object($srch)){
            $note_contact="";
            if(count($srch->Data->data)>0){
                $deal_primarycontact_id = $srch->Data->data[0]->id;

                //Note functionality with duplicate record
                if($email!="") {
                  
                  // if($deal_status==1){

                  // }else {
                    if( $record == 1 ){
                      $this->createContact($merge_vars);
                    }elseif( $record == 2 ){
                      //For updating first record
                        if(empty($merge_vars['mobile'])){
                          unset($merge_vars['mobile']);
                        }
                        if(empty($merge_vars['phone'])){
                          unset($merge_vars['phone']);
                        }
                        if(empty($merge_vars['otherPhone'])){
                          unset($merge_vars['otherPhone']);
                        } 
                        $id = $this->sorting_data_by_id($srch->Data->data);
                        $this->updateContact($merge_vars, $id);
                        $this->createNote($id,$noteFields,'contact');
                      
                    }elseif( $record == 3 ){
                      $id = $this->sorting_data_by_id($srch->Data->data);
                      $this->createNote($id,$noteFields,'contact');
                    }
                    
                  // }
                }
            }

            /*20-12-2016*/
            foreach($srch->Data->data as $chk_contact) {

                array_push($chk_arr,strtolower($chk_contact->email));

            }
        }

        if($email!="") {
          if(!in_array(strtolower($email),$chk_arr))
            {
                $rslt=$this->doRequest($end_point,'POST',$send_salesmate_data,'save');
                if(is_object($rslt)){
                    if($rslt->Status=="success"){
                        $deal_primarycontact_id=$rslt->Data->id;
                        // if($deal_status==1){

                        // }else {
                          $this->createNote($rslt->Data->id,$noteFields,'contact');
                        // }
                    }

                }else{
                    //will have to add log over here
                   global $wpdb;

                    $insertFailContact=$wpdb->insert(
                        $wpdb->prefix.'salesmatelogs',
                        array(
                            'form_id' => $formID,
                            'fromodule' => 'Contacts',
                            'errlogs' => $rslt,
                            'trieddata'=> $send_salesmate_data,
                            'logat' => current_time('mysql', 1)
                        )
                    );

                }
            }

        }else{
            $rslt=$this->doRequest($end_point,'POST',$send_salesmate_data,'save');
            if(is_object($rslt)){

                if($rslt->Status=="success"){
                    $deal_primarycontact_id = $rslt->Data->id;
                    if($deal_status==1){

                    }else {
                      $this->createNote($rslt->Data->id, $noteFields, 'contact');
                    }
                }

            }else{
                //will have to add log over here
                echo $rslt;
                global $wpdb;

                $insertFailContact=$wpdb->insert(
                    $wpdb->prefix.'salesmatelogs',
                    array(
                        'form_id' => $formID,
                        'fromodule' => 'Contacts',
                        'errlogs' => $rslt,
                        'trieddata'=> $send_salesmate_data,
                        'logat' => current_time('mysql', 1)
                    )
                );

            }
        }


     /* Dynamic field rendering */

       /*New deal integration things 5-6-2017*/

       if($deal_status==1){

             $final_deal_array=array();
             $new_company_array=array();
             $new_contact_array=array();
             $deal_merge_array=array();
             $chk_arr_deal=array();
             $deal_title='';
             $endpoint_deal='deal';
             $deal_field=$this->get_field_map_fields($feed, 'dealFields');

             foreach ($deal_field as $name => $fld_id) {

                   if($name=='title'){

                     $deal_title=$this->get_field_value($form,$entry,$fld_id);

                   }

                    if($name=='dealValue'){

                       $deal_value=$this->get_field_value($form,$entry,$fld_id);

                           if (is_numeric($deal_value)) {

                                $deal_merge_array[$name]=(int)$deal_value;

                           } else {

                                $deal_merge_array[$name]=0;

                           }

                   }
                     else{
                         $deal_merge_array[$name]=$this->get_field_value($form,$entry,$fld_id);
                     }

                 }

                 $refine_stage=explode('_',$deal_stage);
                 $deal_merge_array['owner']=(int)$owner;
                 $deal_merge_array['primaryCompany'] = $company_id;
                 $deal_merge_array['primaryContact']=($deal_primarycontact_id!="")?$deal_primarycontact_id:'';
                 $deal_merge_array['pipeline']=$deal_pipeline;
                 $deal_merge_array['stage']=$refine_stage[0];
                 $deal_merge_array['priority']=$deal_priority;
                 $deal_merge_array['status']='Open';
                 $deal_merge_array['source']=$deal_source;
                 $deal_merge_array['currency']=$deal_currency;
                 $deal_merge_array['winProbability']=$deal_winprobability;
                 $deal_data_to_add=json_encode($deal_merge_array);

                //  $deal_search='{"fields":["title"],"query":{"group":{"operator":"AND","rules":[{"condition":"EQUALS","field":{"fieldName":"title","displayName":"Title","type":"Text","groupName":"System Fields","fieldOptions":""},"data":"'.$deal_title.'"}]}}}';
                 $deal_search='{"displayingFields":["deal.id","deal.title"],"filterQuery":{"group":{"operator":"AND","rules":[{"condition":"EQUALS","moduleName":"Deal","field":{"fieldName":"deal.title","displayName":"Title","type":"Text"},"data":"'.$deal_title.'","eventType":"Title"}]}},"sort":{"fieldName":"","order":""},"pipeline":"","moduleId":4,"reportType":"get_data","getRecordsCount":true}';
                 $srch_deal=$this->doRequest('deal','POST',$deal_search,'search');
                 if( !empty( $srch_deal->Data->data ) ){
                  foreach($srch_deal->Data->data as $chk_deal){
                    array_push($chk_arr_deal,$chk_deal->title);
                  }
                 }
                


                 //Make duplicate deal 28-06-2018

                 $result_deal=$this->doRequest('deal', 'POST', $deal_data_to_add, 'save');
                 $this->createNote($result_deal->Data->id, $noteFields, 'deal');
                 if(!is_object($result_deal)){
                    global $wpdb;

                    $insertFailDeal=$wpdb->insert(
                        $wpdb->prefix.'salesmatelogs',
                        array(
                            'form_id' => $formID,
                            'fromodule' => 'Deal',
                            'errlogs' => $result_deal,
                            'trieddata'=> $deal_data_to_add,
                            'logat' => current_time('mysql', 1)
                        )
                    );

                 }


       }

       /*New deal integration things 5-6-2017 end*/
	}

	/**
	 * Custom format the phone type field values before they are returned by $this->get_field_value().
	 *
	 * @param array $entry The Entry currently being processed.
	 * @param string $field_id The ID of the Field currently being processed.
	 * @param GF_Field_Phone $field The Field currently being processed.
	 *
	 * @return string
	 */
	public function get_phone_field_value( $entry, $field_id, $field ) {

		// Get the field value from the Entry Object.
		$field_value = rgar( $entry, $field_id );

		// If there is a value and the field phoneFormat setting is set to standard reformat the value.
		if ( ! empty( $field_value ) && $field->phoneFormat == 'standard' && preg_match( '/^\D?(\d{3})\D?\D?(\d{3})\D?(\d{4})$/', $field_value, $matches ) ) {
			$field_value = sprintf( '%s-%s-%s', $matches[1], $matches[2], $matches[3] );
		}

		return $field_value;
	}
    // Sorting data by id 
    public function sorting_data_by_id( $search_data ){
      $data_ids = array();
      foreach( $search_data as $key => $value ){
        $data_ids[] = $value->id;
      }
      
      rsort($data_ids);
      return $data_ids[0];
     }

    public function delay_feed( $feed, $entry, $form ) {
      return;
   }

	// # ADMIN FUNCTIONS -----------------------------------------------------------------------------------------------

	/**
	 * Creates a custom page for this add-on.
	 */
	public function plugin_page() {
		echo 'This page appears in the Forms menu';
	}

	/**
	 * Configures the settings which should be rendered on the add-on settings tab.
	 *
	 * @return array
	 */
	public function plugin_settings_fields() {
		return array(

      array(

          'title' => esc_html__('Salemate Add-On Settings', 'gravityformsalesmate'),

          'fields' => array(

              array(
                  'name' => 'domainname',
                  'tooltip' => esc_html__('Your domain like https://example.salesmate.io', 'gravityformsalesmate'),
                  'label' => esc_html__('Your salemate domain', 'gravityformsalesmate'),
                  'type' => 'text',
                  'placeholder'=>'Ex:  https://example.salesmate.io',
                  'class' => 'medium',
                  'feedback_callback'=>array(&$this, 'is_invalid_api')
              ),

              array(
                  'name' => 'appaccesskey',
                  'tooltip' => esc_html__('Add App access key from your account', 'gravityformsalesmate'),
                  'label' => esc_html__('App accesskey', 'gravityformsalesmate'),
                  'type' => 'text',
                  'class' => 'medium',
                  'feedback_callback'=>array(&$this, 'is_invalid_api')
              ),

              array(
                  'name' => 'appprivatekey',
                  'tooltip' => esc_html__('Add app private key from your salesmate account', 'gravityformsalesmate'),
                  'label' => esc_html__('App private key', 'gravityformsalesmate'),
                  'type' => 'text',
                  'class' => 'medium',
                  'feedback_callback'=>array(&$this, 'is_invalid_api')
              ),

              array(
                  'name' => 'sessiontoken',
                  'tooltip' => esc_html__('Add session token from your salesmate account', 'gravityformsalesmate'),
                  'label' => esc_html__('Session token', 'gravityformsalesmate'),
                  'type' => 'text',
                  'class' => 'medium',
                  'feedback_callback'=>array(&$this, 'is_invalid_api')
              )

          ),
      ),
  );
}

	/**
	 * Configures the settings which should be rendered on the feed edit page in the Form Settings > Simple Feed Add-On area.
	 *
	 * @return array
	 */
	public function feed_settings_fields() {
    if(isset($_POST['_gaddon_setting_deal'])){
      if($_POST['_gaddon_setting_deal']==1){

              $req=1;

      }else{

              $req=0;
      }
    }else {
        $req=0;
    }

    $plugin_setting=get_option("gravityformsaddon_gravityformsalesmate_settings");
		return array(
			array(
				'title' => esc_html__('Salesmate feed settings', 'gravityformsalesmate'),
        'fields' => array(
                array(
                  'label' => esc_html__( 'Domain name', 'gravityformsalesmate' ),
                  'type'  => 'hidden',
                  'name'  => 'domainName',
                  'class'=>'domainaname',
                  'chkdomain'=>$plugin_setting['domainname'],
                  'value' =>$plugin_setting['domainname']
              ),
              array(
                  'label' => esc_html__( 'Access Key', 'gravityformsalesmate' ),
                  'type'  => 'hidden',
                  'name'  => 'appaccesskey',
                  'access-key'=>$plugin_setting['appaccesskey'],
                  'class'=>'app_accesskey',
                  'value' =>$plugin_setting['appaccesskey']
              ),
              array(
                  'label' => esc_html__( 'Private Key', 'gravityformsalesmate' ),
                  'type'  => 'hidden',
                  'name'  => 'privatekey',
                  'privat-key'=>$plugin_setting['appprivatekey'],
                  'class'=>'app_privatekey',
                  'value' =>$plugin_setting['appprivatekey']
              ),
              array(
                  'label' => esc_html__( 'Session Token', 'gravityformsalesmate' ),
                  'type'  => 'hidden',
                  'name'  => 'sessiontoken',
                  'token'=>$plugin_setting['sessiontoken'],
                  'class'=>'app_session',
                  'value' =>$plugin_setting['sessiontoken']
              ),
              array(
                  'label'=>esc_html__('Pipeline Index','gravityformsalesmate'),
                  'type'=>'hidden',
                  'name'=>'pipeIndex',
                  'class'=>'pipeline_index',
              ),

              array(
                  'label'=>esc_html__('Pipeline Stages','gravityformsalesmate'),
                  'type'=>'hidden',
                  'name'=>'pipeStages',
                  'class'=>'pipeline_stages',
              ),
              array(
                'label' => esc_html__('Feed name', 'gravityformsalesmate'),
                'type' => 'text',
                'name' => 'feedName',
                'tooltip' => esc_html__('This is the tooltip', 'gravityformsalesmate'),
                'class' => 'small',
            ),
            array(
              'label' => esc_html__('WHEN A DUPLICATE RECORD IS FOUND', 'gravityformsalesmate'),
              'type' => 'select',
              'name' => 'record',
              'class' => 'medium dealfield deal_source',
              'choices'=>$this->getDataChange(),
            ),
            array(
                'type' => 'select',
                'label' => __('Owner', 'gravityformsalesmate'),
                'name' => 'owner',
                'default_value' => '',
                'choices' =>$this->ListOwners()
            ),
            array(
              'name' => 'mappedFields',
              'label' => esc_html__('Contact Fields', 'gravityformsalesmate'),
              'type' => 'field_map',
              'field_map' => $this->ListField()
            ),
            array(
                'label'   => esc_html__('Enable Deal?', 'simpleaddon'),
                'type'    => 'checkbox',
                'name'    => 'dealmode',
                'class' =>'deal_apply',
                'tooltip' => esc_html__('If Deal checked then deal module will enabled.', 'simpleaddon'),
                'choices' => array(

                    array(
                        'label' => esc_html__( 'Deal', 'simpleaddon' ),
                        'name'  => 'deal',
                    ),

                ),
            ),
            array(
                'name' => 'dealFields',
                'label' => esc_html__('Deal Fields', 'gravityformsalesmate'),
                'type' => 'field_map',
                'field_map' => $this->ListDealField()
            ),
            array(
                'label' => esc_html__('Win Probability', 'gravityformsalesmate'),
                'type' => 'text',
                'name' => 'winProbability',
                'tooltip' => esc_html__('Allow only numbers and floating point value', 'gravityformsalesmate'),
                'class' => 'medium dealfield deal_prob',
                'placeholder'=>'Enter Number only',
                'required'=>$req
            ),
            array(
                'label'=>esc_html__('Pipeline', 'gravityformsalesmate'),
                'type'=>'select',
                'name'=>'pipeline',
                'tooltip'=>esc_html__('Choose deal pipeline', 'gravityformsalesmate'),
                'class'=>'medium dealfield deal_pipeline',
                'choices'=>$this->getPilineStages("apps/dealPipeline","pipeline"),
                'required'=>$req
            ),
            array(
                'label' => esc_html__('Stage', 'gravityformsalesmate'),
                'type' => 'select',
                'name' => 'stage',
                'tooltip' => esc_html__('Stage must be from "Qualified,Contact Made,Proposal Presented,Quote Sent,In Negotiation"', 'gravityformsalesmate'),
                'class' => 'medium dealfield deal_stage',
                'choices'=>$this->getPilineStages("apps/dealPipeline","stage"),
                'required'=>$req
            ),
            array(
                'label' => esc_html__('Priority', 'gravityformsalesmate'),
                'type' => 'select',
                'name' => 'priority',
                'tooltip' => esc_html__('Priority must be from "High,Medium,Low"', 'gravityformsalesmate'),
                'class' => 'medium dealfield deal_priority',
                'choices'=>$this->getPriority(),
                'required'=>$req
            ),
            array(
                'label' => esc_html__('Source', 'gravityformsalesmate'),
                'type' => 'select',
                'name' => 'source',
                'tooltip' => esc_html__('Choose source from selectbox', 'gravityformsalesmate'),
                'class' => 'medium dealfield deal_source',
                'choices'=>$this->getSource(),
                'required'=>$req
            ),
            array(
              'label' => esc_html__('Currency', 'gravityformsalesmate'),
              'type' => 'select',
              'name' => 'currency',
              'tooltip' => esc_html__('Choose currency code from selectbox', 'gravityformsalesmate'),
              'class' => 'medium dealfield deal_currency',
              // 'placeholder'=>'Enter currency format like INR,INP,ILS',
              'choices'=>$this->getCurrency(),
              'required'=>$req
            ),
					array(
						'name'           => 'condition',
						'label'          => esc_html__( 'Condition', 'gravityformsalesmate' ),
						'type'           => 'feed_condition',
						'checkbox_label' => esc_html__( 'Enable Condition', 'gravityformsalesmate' ),
						'instructions'   => esc_html__( 'Process this simple feed if', 'gravityformsalesmate' ),
					),
				),
			),
		);
	}
  //getting currency
  public function getCurrency(){

    $currency=$this->doRequest('lookups/active/currency','GET','false','currency');
    
    if(is_object($currency)){
      $currency_select_options[] = array(
        'name' =>'',
        'value' =>'',
        'label' => __('Select Currency','gravityformsalesmate'),
      );
      for($k=0; $k<count($currency->Data); $k++){
        $currency_select_options[]=array(
          'name' =>$currency->Data[$k]->name,
          'value' =>$currency->Data[$k]->code,
          'label' => __($currency->Data[$k]->displayLabel,'gravityformsalesmate'),
         );
      }

    }
    return $currency_select_options;
  }
	/**
	 * Configures which columns should be displayed on the feed list page.
	 *
	 * @return array
	 */
	public function feed_list_columns() {
		return array(
			'feedName'  => esc_html__( 'Name', 'gravityformsalesmate' ),
			'mytextbox' => esc_html__( 'My Textbox', 'gravityformsalesmate' ),
		);
	}

  /*

      * All Api call will Manage from this doRequest.

      * Need to pass Argument as per requirement.

      * @Returns Array or status Digit.

     */

    public function doRequest($endpoint=false,$method,$postdata=false,$type=''){
      $ch=curl_init();

      if( $type == 'Auth' ){
        $endpoint_val=$this->getApiDomain($endpoint);
      }else if( $type == 'contact' && $method == 'PUT' ){
        $endpoint_val=$this->getApiDomain($endpoint,$type,$method);
      }else{
        $endpoint_val=$this->getApiDomain($endpoint,$type);
      }
     
      curl_setopt($ch,CURLOPT_URL,str_replace(' ','%20',$endpoint_val));
      curl_setopt($ch,CURLOPT_CUSTOMREQUEST,$method);
      curl_setopt($ch, CURLOPT_RETURNTRANSFER, 1);
      curl_setopt($ch, CURLOPT_HTTPHEADER, $this->header_data());

      if ( OPENSSL_VERSION_NUMBER >= 0x1000100f ) {

        if ( ! defined( 'CURL_SSLVERSION_TLSv1_2' ) ) {

          define( 'CURL_SSLVERSION_TLSv1_2', 6 );

        }

        curl_setopt( $ch, CURLOPT_SSLVERSION, CURL_SSLVERSION_TLSv1_2 );

      }else {

          if ( ! defined( 'CURL_SSLVERSION_TLSv1' ) ) {

            define( 'CURL_SSLVERSION_TLSv1', 1 );

          }

          curl_setopt( $ch, CURLOPT_SSLVERSION, CURL_SSLVERSION_TLSv1 );
      }

      if(($method=='POST' || $method=='PUT') && $type!='Auth'){

          curl_setopt($ch, CURLOPT_POST, 1);
          curl_setopt($ch, CURLOPT_POSTFIELDS, $postdata);

      }

      $resp=curl_exec($ch);
  
      $dir = plugin_dir_path( __FILE__ );

      $fp=fopen($dir.'logs/log.txt', 'w');

      if($type!='Auth'){

        fwrite($fp, $resp);

      }

      fclose($fp);

      if(!$resp)
      {
          $msg_status=0;

      }else if(curl_getinfo($ch, CURLINFO_HTTP_CODE)=='404'){

          $msg_status=1;

      }else if(curl_getinfo($ch, CURLINFO_HTTP_CODE)=='200'){

          $msg_status=json_decode($resp);

      } else {

          $msg_status=$resp;

      }

  return $msg_status;

      curl_close();

  }

  
    /*
     * is_invalide_api to check credential is valid or not.
     * @returns boolean value.
    */
    public function is_invalid_api(){
      $status=$this->doRequest('core/v4/users?status=active','GET',false,'Auth');
		
  		if(is_object($status)){

          if($status->Status == 'success')
          {
                return true;

          }else {

				return false;
          }

       }else {
             return false;
       }

    }

      /*
    * just provide End-point and what you require from "Pipeline" and "Stage"
    * @return array
    */
    public function getPilineStages($end_point,$requiredatatype){
      $pieplinestagedefault=$this->doRequest($end_point,'GET',false,'getDealPipeline');
      if($requiredatatype=='pipeline'){
        for($k=0; $k<count($pieplinestagedefault->Data); $k++){
              $stages_select_options[]=array(
                'name' =>$pieplinestagedefault->Data[$k]->pipeline,
                'value' =>$pieplinestagedefault->Data[$k]->pipeline,
                'label' => __($pieplinestagedefault->Data[$k]->pipeline,'gravityformsalesmate')
               );
         }
      }else {
        
        for($k=0; $k<count($pieplinestagedefault->Data); $k++){
          for($ps=0;$ps<count($pieplinestagedefault->Data[$k]->stages);$ps++){
            $stages_select_options[]=array(
              'name' =>$pieplinestagedefault->Data[$k]->stages[$ps]->stage,
              'value' =>$pieplinestagedefault->Data[$k]->stages[$ps]->stage.'_'.$k,
              'label' => __($pieplinestagedefault->Data[$k]->stages[$ps]->stage,'gravityformsalesmate')
             );
          }         
         }

      }

      return $stages_select_options;

    }

    
    public function getSource(){

      $dealSource=$this->doRequest('deals/getEditableFields','GET','false','dealsource');
      if(is_object($dealSource)){
        $sourceOptions = array();
        $dataSourece=array_filter($dealSource->Data, function($obj){
          if($obj->fieldName == 'source') {
            return true;
          }
        });
        $optionssource = array_values($dataSourece);
        $optionsData = json_decode($optionssource[0]->fieldOptions);
        $sourceOptions['source'] = $optionsData->values;
        for($k=0; $k<count($optionsData->values); $k++){
          $source_select_options[]=array(
            'name' =>$optionsData->values[$k],
            'value' =>$optionsData->values[$k],
            'label' => __($optionsData->values[$k],'gravityformsalesmate'),
           );
        }

      }
      return $source_select_options;
    }

    public function getDataChange(){
      $dataOptions = array();
      
      $dataOptions[1]['name'] = "insert_record";
      $dataOptions[1]['value'] = 1;
      $dataOptions[1]['label'] = "Always create a new record";

      
      $dataOptions[2]['name'] = "update_record";
      $dataOptions[2]['value'] = 2;
      $dataOptions[2]['label'] = "Update an existing record";

      $dataOptions[0]['name'] = "create_note";
      $dataOptions[0]['value'] = 3;
      $dataOptions[0]['label'] = "Place a note in the timeline";

        for($k=0; $k<count($dataOptions); $k++){
          $source_select_options[]=array(
            'name' =>$dataOptions[$k]['name'],
            'value' =>$dataOptions[$k]['value'],
            'label' => __($dataOptions[$k]['label'],'datachange'),
           );
        }
      return $source_select_options;
    }

    public function getPriority(){
      $dealPriority=$this->doRequest('deals/getEditableFields','GET','false','dealsource');
      
      if(is_object($dealPriority)){
        $priorityOptions = array();
        $dataSourece=array_filter($dealPriority->Data, function($obj){
          if($obj->fieldName == 'priority') {
            return true;
          }
        });
        
        $optionspriority = array_values($dataSourece);
        $optionsData = json_decode($optionspriority[0]->fieldOptions);
        $sourceOptions['source'] = $optionsData->values;
        
        for($k=0; $k<count($optionsData->values); $k++){
          $priority_select_options[]=array(
            'name' =>$optionsData->values[$k],
            'value' =>$optionsData->values[$k],
            'label' => __($optionsData->values[$k],'gravityformsalesmate'),
           );
        }

      }
      return $priority_select_options;
    }

    
     /*
     * To get fields of deal salesmate which listed to map with gravity form's field.
     *
     * @Return array.
     */
    public function ListDealField() {

      $end_point = 'deals/getEditableFields';

      $all_fields=$this->doRequest($end_point,'GET',false,'Fields');

      if(is_object($all_fields)) {

              $all_listfield = $all_fields->Data;
              $data_field = array();
              $displayDealFields = array();
              $arrayIgnore = array("primaryCompany","primaryContact","owner","priority","pipeline","stage","status","source","currency","winProbability");
              
              foreach ($all_listfield as $value_field) {

                  // if ($value_field->fieldName!='primaryContact' && $value_field->fieldName!='primaryCompany' && $value_field->type != 'Date' && $value_field->type != 'DateTime' && $value_field->fieldName != 'owner' && $value_field->type!='Select' && $value_field->type!='Percentage' && $value_field->fieldName!='currency') {
                  if(!in_array($value_field->fieldName, $arrayIgnore) &&  !in_array($value_field->type, $arrayIgnore)){
                    if(isset($_POST['_gaddon_setting_deal'])){

                      if($_POST['_gaddon_setting_deal']==1){

                          $data_field[] = array(

                              'name' => $value_field->fieldName,
                              'label' => esc_html__($value_field->displayName, 'gravityformsalesmate'),
                              'required' => $value_field->isRequired,

                          );

                      } else {

                        $data_field[] = array(

                          'name' => $value_field->fieldName,
                          'label' => esc_html__($value_field->displayName, 'gravityformsalesmate'),
                          'required' => 0,

                        );

                      }
                    }else {
                      $data_field[] = array(

                        'name' => $value_field->fieldName,
                        'label' => esc_html__($value_field->displayName, 'gravityformsalesmate'),
                        'required' => 0,

                      );
                    }

                  }

              }

         return $data_field;

     }

  }

  /*
     * To get fields of contact salesmate which listed to map with gravity form's field.
     *
     * @Return array.
     */
    public function ListField() {

      $end_point = 'contacts/getEditableFields';

      $all_fields=$this->doRequest($end_point,'GET',false,'Fields');

      $contactFields = array();
      $arrayIgnoreContact = array("owner","currency","company");

      if(is_object($all_fields)){

              $all_listfield = $all_fields->Data;
              $data_field = array();

              foreach ($all_listfield as $value_field) {

                  if(!in_array($value_field->fieldName, $arrayIgnoreContact) && !in_array($value_field->type, $arrayIgnoreContact)){
                  // if ($value_field->type != 'Boolean' && $value_field->type != 'Lookup' && $value_field->fieldName != 'owner') {

                    if($value_field->fieldName == 'name'){

                      $data_field[] = array(
                        'name' => $value_field->fieldName,
                        'label' => esc_html__($value_field->displayName, 'gravityformsalesmate'),
                        'required' =>0,
                      );

                    }else {

                      $data_field[] = array(
                        'name' => $value_field->fieldName,
                        'label' => esc_html__($value_field->displayName, 'gravityformsalesmate'),
                        'required' => $value_field->isRequired,
                      );

                    }

                  }

              }

              $data_field[]=array(
                  'name'=>'company',
                  'label'=>esc_html__('Company','gravityformsalesmate'),
                  'required'=>0
              );

         return $data_field;

     }

  }

  
    /*
     * To get all active users which stated as owner.
     *
     * @Return array.
    */
    public function ListOwners() {

      $end_point = "core/v4/users?status=active";
      $ch = curl_init();

      $all_users=$this->doRequest($end_point,'GET',false,'Owners');

      if(is_object($all_users))
      {

          for ($j = 0; $j < count($all_users->Data); $j++) {

              $select_option[] = array(
                  'name' => $all_users->Data[$j]->firstName . '' . $all_users->Data[$j]->lastName,
                  'value' => $all_users->Data[$j]->id,
                  'label' => __($all_users->Data[$j]->firstName . " " . $all_users->Data[$j]->lastName,'gravityformsalesmate')
              );

          }

      }else{

          $select_option[]='Nothing';

      }

     return $select_option;

  }



	/**
	 * Format the value to be displayed in the mytextbox column.
	 *
	 * @param array $feed The feed being included in the feed list.
	 *
	 * @return string
	 */
	public function get_column_value_mytextbox( $feed ) {
		return '<b>' . rgars( $feed, 'meta/mytextbox' ) . '</b>';
	}

	/**
	 * Prevent feeds being listed or created if an api key isn't valid.
	 *
	 * @return bool
	 */
	public function can_create_feed() {

		// Get the plugin settings.
		$settings = $this->get_plugin_settings();

		// Access a specific setting e.g. an api key
		$key = rgar( $settings, 'apiKey' );

		return true;
	}

  //JQUERY

     /*
    * Add style and script
    */
    public function scripts($form = '', $is_ajax = false) {

      $scripts = array(
        array(
          'handle'  => 'my_script_js',
          'src'     => $this->get_base_url() . '/js/my_script.js',
          'version' => $this->_version,
          'deps'    => array( 'jquery' ),
          'strings' => array(
            'first'  => esc_html__( 'First Choice', 'simplefeedaddon' ),
            'second' => esc_html__( 'Second Choice', 'simplefeedaddon' ),
            'third'  => esc_html__( 'Third Choice', 'simplefeedaddon' ),
          ),
          'enqueue' => array(
            array(
              'admin_page' => array( 'form_settings' ),
              'tab'        => 'gravityformsalesmate',
            ),
          ),
        ),
      );
  
      return array_merge( parent::scripts(), $scripts );
    }

    

	/**
	 * Return the stylesheets which should be enqueued.
	 *
	 * @return array
	 */
	public function styles() {

		$styles = array(
			array(
				'handle'  => 'my_styles_css',
				'src'     => $this->get_base_url() . '/css/my_styles.css',
				'version' => $this->_version,
				'enqueue' => array(
					array( 'field_types' => array( 'poll' ) ),
				),
			),
		);

		return array_merge( parent::styles(), $styles );
	}


    public function dump_values($value){
      echo "<pre>";
          print_r($value);
          echo "<pre>";
          exit;
    }
}
